# This file has been generated


build_system = 'AppVeyor'
build_id = '33297084'
build_number = '663'
build_version = '1.0.663'
job_id = 'dmir2rlxs60ywhy0'
job_url = 'https://ci.appveyor.com/project/KyleAltendorf/pm/build/job/dmir2rlxs60ywhy0'
