# This file was automatically generated by sub\epyqlib\epyqlib\cli\generateversion.py
__version_build_time__ = '2020-06-03T17:07:09.659628'

__version__ = '2020.6'
__sha__ = '995791cd6f9fa07af75d4e30639b34ec0db74218'
__revision__ = '995791cd6f9fa07af75d4e30639b34ec0db74218'
