# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'mainwindow.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1200, 900)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.splitter = QtWidgets.QSplitter(self.centralwidget)
        self.splitter.setOrientation(QtCore.Qt.Horizontal)
        self.splitter.setObjectName("splitter")
        self.gridLayoutWidget = QtWidgets.QWidget(self.splitter)
        self.gridLayoutWidget.setObjectName("gridLayoutWidget")
        self.gridLayout = QtWidgets.QGridLayout(self.gridLayoutWidget)
        self.gridLayout.setContentsMargins(0, 0, 0, 0)
        self.gridLayout.setObjectName("gridLayout")
        self.parameter_view = QtWidgets.QTreeView(self.gridLayoutWidget)
        self.parameter_view.setAlternatingRowColors(True)
        self.parameter_view.setObjectName("parameter_view")
        self.gridLayout.addWidget(self.parameter_view, 2, 0, 1, 1)
        self.parameter_search_box = SearchBox(self.gridLayoutWidget)
        self.parameter_search_box.setObjectName("parameter_search_box")
        self.gridLayout.addWidget(self.parameter_search_box, 0, 0, 1, 1)
        self.tabWidget = QtWidgets.QTabWidget(self.splitter)
        self.tabWidget.setObjectName("tabWidget")
        self.tab_2 = QtWidgets.QWidget()
        self.tab_2.setObjectName("tab_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.tab_2)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.can_view = QtWidgets.QTreeView(self.tab_2)
        self.can_view.setAlternatingRowColors(True)
        self.can_view.setObjectName("can_view")
        self.gridLayout_2.addWidget(self.can_view, 1, 0, 1, 1)
        self.can_search_box = SearchBox(self.tab_2)
        self.can_search_box.setObjectName("can_search_box")
        self.gridLayout_2.addWidget(self.can_search_box, 0, 0, 1, 1)
        self.horizontalLayout_2.addLayout(self.gridLayout_2)
        self.tabWidget.addTab(self.tab_2, "")
        self.tab_21 = QtWidgets.QWidget()
        self.tab_21.setObjectName("tab_21")
        self.horizontalLayout_21 = QtWidgets.QHBoxLayout(self.tab_21)
        self.horizontalLayout_21.setObjectName("horizontalLayout_21")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.sunspec_view = QtWidgets.QTreeView(self.tab_21)
        self.sunspec_view.setObjectName("sunspec_view")
        self.gridLayout_3.addWidget(self.sunspec_view, 1, 0, 1, 1)
        self.sunspec_search_box = SearchBox(self.tab_21)
        self.sunspec_search_box.setObjectName("sunspec_search_box")
        self.gridLayout_3.addWidget(self.sunspec_search_box, 0, 0, 1, 1)
        self.horizontalLayout_21.addLayout(self.gridLayout_3)
        self.tabWidget.addTab(self.tab_21, "")
        self.tab = QtWidgets.QWidget()
        self.tab.setObjectName("tab")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.tab)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.gridLayout_4 = QtWidgets.QGridLayout()
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.value_set_view = QtWidgets.QTreeView(self.tab)
        self.value_set_view.setObjectName("value_set_view")
        self.gridLayout_4.addWidget(self.value_set_view, 1, 0, 1, 1)
        self.value_set_search_box = SearchBox(self.tab)
        self.value_set_search_box.setObjectName("value_set_search_box")
        self.gridLayout_4.addWidget(self.value_set_search_box, 0, 0, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout_4)
        self.tabWidget.addTab(self.tab, "")
        self.gridLayoutWidget22 = QtWidgets.QWidget(self.splitter)
        self.gridLayoutWidget22.setObjectName("gridLayoutWidget22")
        self.gridLayout22 = QtWidgets.QGridLayout(self.gridLayoutWidget22)
        self.gridLayout22.setContentsMargins(0, 0, 0, 0)
        self.gridLayout22.setObjectName("gridLayout22")
        self.check_result_view = QtWidgets.QTreeView(self.gridLayoutWidget22)
        self.check_result_view.setAlternatingRowColors(True)
        self.check_result_view.setObjectName("check_result_view")
        self.gridLayout22.addWidget(self.check_result_view, 2, 0, 1, 1)
        self.check_result_search_box = SearchBox(self.gridLayoutWidget22)
        self.check_result_search_box.setObjectName("check_result_search_box")
        self.gridLayout22.addWidget(self.check_result_search_box, 0, 0, 1, 1)
        self.gridLayout_5.addWidget(self.splitter, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1200, 30))
        self.menubar.setObjectName("menubar")
        self.menu_File = QtWidgets.QMenu(self.menubar)
        self.menu_File.setObjectName("menu_File")
        self.action_new_value_set_2 = QtWidgets.QMenu(self.menu_File)
        self.action_new_value_set_2.setObjectName("action_new_value_set_2")
        self.menu_Project = QtWidgets.QMenu(self.menu_File)
        self.menu_Project.setObjectName("menu_Project")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        self.menuTools = QtWidgets.QMenu(self.menubar)
        self.menuTools.setObjectName("menuTools")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.action_open_project = QtWidgets.QAction(MainWindow)
        self.action_open_project.setObjectName("action_open_project")
        self.action_save_as_project = QtWidgets.QAction(MainWindow)
        self.action_save_as_project.setObjectName("action_save_as_project")
        self.action_save_project = QtWidgets.QAction(MainWindow)
        self.action_save_project.setObjectName("action_save_project")
        self.action_new_project = QtWidgets.QAction(MainWindow)
        self.action_new_project.setObjectName("action_new_project")
        self.action_about = QtWidgets.QAction(MainWindow)
        self.action_about.setObjectName("action_about")
        self.action_import_sym = QtWidgets.QAction(MainWindow)
        self.action_import_sym.setObjectName("action_import_sym")
        self.action_export_sym = QtWidgets.QAction(MainWindow)
        self.action_export_sym.setObjectName("action_export_sym")
        self.action_Project = QtWidgets.QAction(MainWindow)
        self.action_Project.setObjectName("action_Project")
        self.action_new_value_set = QtWidgets.QAction(MainWindow)
        self.action_new_value_set.setObjectName("action_new_value_set")
        self.action_Value_Set = QtWidgets.QAction(MainWindow)
        self.action_Value_Set.setObjectName("action_Value_Set")
        self.action_save_value_set = QtWidgets.QAction(MainWindow)
        self.action_save_value_set.setObjectName("action_save_value_set")
        self.action_save_as_value_set = QtWidgets.QAction(MainWindow)
        self.action_save_as_value_set.setObjectName("action_save_as_value_set")
        self.action_open_value_set = QtWidgets.QAction(MainWindow)
        self.action_open_value_set.setObjectName("action_open_value_set")
        self.action_import_smdx = QtWidgets.QAction(MainWindow)
        self.action_import_smdx.setObjectName("action_import_smdx")
        self.action_full_import = QtWidgets.QAction(MainWindow)
        self.action_full_import.setObjectName("action_full_import")
        self.action_full_export = QtWidgets.QAction(MainWindow)
        self.action_full_export.setObjectName("action_full_export")
        self.action_full_export_first_time = QtWidgets.QAction(MainWindow)
        self.action_full_export_first_time.setObjectName("action_full_export_first_time")
        self.action_check = QtWidgets.QAction(MainWindow)
        self.action_check.setObjectName("action_check")
        self.action_new_value_set_2.addAction(self.action_new_value_set)
        self.action_new_value_set_2.addAction(self.action_open_value_set)
        self.action_new_value_set_2.addAction(self.action_save_value_set)
        self.action_new_value_set_2.addAction(self.action_save_as_value_set)
        self.menu_Project.addAction(self.action_new_project)
        self.menu_Project.addAction(self.action_open_project)
        self.menu_Project.addAction(self.action_save_project)
        self.menu_Project.addAction(self.action_save_as_project)
        self.menu_File.addAction(self.menu_Project.menuAction())
        self.menu_File.addAction(self.action_new_value_set_2.menuAction())
        self.menu_File.addSeparator()
        self.menu_File.addAction(self.action_import_smdx)
        self.menu_File.addAction(self.action_full_export)
        self.menuHelp.addAction(self.action_about)
        self.menuTools.addAction(self.action_check)
        self.menubar.addAction(self.menu_File.menuAction())
        self.menubar.addAction(self.menuTools.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(0)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_2), _translate("MainWindow", "CAN"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab_21), _translate("MainWindow", "SunSpec"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.tab), _translate("MainWindow", "Value Set"))
        self.menu_File.setTitle(_translate("MainWindow", "&File"))
        self.action_new_value_set_2.setTitle(_translate("MainWindow", "&Value Set"))
        self.menu_Project.setTitle(_translate("MainWindow", "&Project"))
        self.menuHelp.setTitle(_translate("MainWindow", "&Help"))
        self.menuTools.setTitle(_translate("MainWindow", "&Tools"))
        self.action_open_project.setText(_translate("MainWindow", "&Open..."))
        self.action_save_as_project.setText(_translate("MainWindow", "Save &As..."))
        self.action_save_project.setText(_translate("MainWindow", "&Save"))
        self.action_save_project.setShortcut(_translate("MainWindow", "Ctrl+S"))
        self.action_new_project.setText(_translate("MainWindow", "&New"))
        self.action_about.setText(_translate("MainWindow", "&About..."))
        self.action_import_sym.setText(_translate("MainWindow", "&Import .sym..."))
        self.action_export_sym.setText(_translate("MainWindow", "&Export .sym..."))
        self.action_Project.setText(_translate("MainWindow", "&Project"))
        self.action_new_value_set.setText(_translate("MainWindow", "&New"))
        self.action_Value_Set.setText(_translate("MainWindow", "&Value Set..."))
        self.action_save_value_set.setText(_translate("MainWindow", "&Save"))
        self.action_save_as_value_set.setText(_translate("MainWindow", "Save &As..."))
        self.action_open_value_set.setText(_translate("MainWindow", "&Open..."))
        self.action_import_smdx.setText(_translate("MainWindow", "Import &SMDX..."))
        self.action_full_import.setText(_translate("MainWindow", "Full import..."))
        self.action_full_export.setText(_translate("MainWindow", "&Full Export..."))
        self.action_full_export_first_time.setText(_translate("MainWindow", "Full Export (First Time)..."))
        self.action_check.setText(_translate("MainWindow", "&Check"))
from epyqlib.searchbox import SearchBox
