# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'importexportdialog.ui'
#
# Created by: PyQt5 UI code generator 5.13.0
#
# WARNING! All changes made in this file will be lost!


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(708, 646)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Dialog.sizePolicy().hasHeightForWidth())
        Dialog.setSizePolicy(sizePolicy)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.pick_sunspec_tables_c = QtWidgets.QPushButton(Dialog)
        self.pick_sunspec_tables_c.setObjectName("pick_sunspec_tables_c")
        self.gridLayout.addWidget(self.pick_sunspec_tables_c, 3, 3, 1, 1)
        self.label_2 = QtWidgets.QLabel(Dialog)
        self.label_2.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 1, 0, 1, 1)
        self.sunspec_tables_c = QtWidgets.QLineEdit(Dialog)
        self.sunspec_tables_c.setObjectName("sunspec_tables_c")
        self.gridLayout.addWidget(self.sunspec_tables_c, 3, 2, 1, 1)
        self.sil_c_label = QtWidgets.QLabel(Dialog)
        self.sil_c_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sil_c_label.setObjectName("sil_c_label")
        self.gridLayout.addWidget(self.sil_c_label, 8, 0, 1, 1)
        self.pick_sil_c = QtWidgets.QPushButton(Dialog)
        self.pick_sil_c.setObjectName("pick_sil_c")
        self.gridLayout.addWidget(self.pick_sil_c, 8, 3, 1, 1)
        self.hierarchy = QtWidgets.QLineEdit(Dialog)
        self.hierarchy.setObjectName("hierarchy")
        self.gridLayout.addWidget(self.hierarchy, 1, 2, 1, 1)
        self.label_3 = QtWidgets.QLabel(Dialog)
        self.label_3.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 5, 0, 1, 1)
        self.can = QtWidgets.QLineEdit(Dialog)
        self.can.setObjectName("can")
        self.gridLayout.addWidget(self.can, 0, 2, 1, 1)
        self.tables_c_label = QtWidgets.QLabel(Dialog)
        self.tables_c_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.tables_c_label.setObjectName("tables_c_label")
        self.gridLayout.addWidget(self.tables_c_label, 2, 0, 1, 1)
        self.sil_c = QtWidgets.QLineEdit(Dialog)
        self.sil_c.setObjectName("sil_c")
        self.gridLayout.addWidget(self.sil_c, 8, 2, 1, 1)
        self.spreadsheet_user = QtWidgets.QLineEdit(Dialog)
        self.spreadsheet_user.setObjectName("spreadsheet_user")
        self.gridLayout.addWidget(self.spreadsheet_user, 6, 2, 1, 1)
        self.spreadsheet = QtWidgets.QLineEdit(Dialog)
        self.spreadsheet.setObjectName("spreadsheet")
        self.gridLayout.addWidget(self.spreadsheet, 5, 2, 1, 1)
        self.remove_smdx = QtWidgets.QPushButton(Dialog)
        self.remove_smdx.setObjectName("remove_smdx")
        self.gridLayout.addWidget(self.remove_smdx, 12, 3, 1, 1)
        self.pick_spreadsheet = QtWidgets.QPushButton(Dialog)
        self.pick_spreadsheet.setObjectName("pick_spreadsheet")
        self.gridLayout.addWidget(self.pick_spreadsheet, 5, 3, 1, 1)
        self.pick_smdx = QtWidgets.QPushButton(Dialog)
        self.pick_smdx.setObjectName("pick_smdx")
        self.gridLayout.addWidget(self.pick_smdx, 11, 3, 1, 1)
        self.pick_spreadsheet_user = QtWidgets.QPushButton(Dialog)
        self.pick_spreadsheet_user.setObjectName("pick_spreadsheet_user")
        self.gridLayout.addWidget(self.pick_spreadsheet_user, 6, 3, 1, 1)
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.sunspec_tables_c_label = QtWidgets.QLabel(Dialog)
        self.sunspec_tables_c_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sunspec_tables_c_label.setObjectName("sunspec_tables_c_label")
        self.gridLayout.addWidget(self.sunspec_tables_c_label, 3, 0, 1, 1)
        self.tables_c = QtWidgets.QLineEdit(Dialog)
        self.tables_c.setObjectName("tables_c")
        self.gridLayout.addWidget(self.tables_c, 2, 2, 1, 1)
        self.pick_tables_c = QtWidgets.QPushButton(Dialog)
        self.pick_tables_c.setObjectName("pick_tables_c")
        self.gridLayout.addWidget(self.pick_tables_c, 2, 3, 1, 1)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.from_directory = QtWidgets.QPushButton(Dialog)
        self.from_directory.setObjectName("from_directory")
        self.horizontalLayout.addWidget(self.from_directory)
        self.buttons = QtWidgets.QDialogButtonBox(Dialog)
        self.buttons.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttons.setObjectName("buttons")
        self.horizontalLayout.addWidget(self.buttons)
        self.gridLayout.addLayout(self.horizontalLayout, 14, 0, 1, 4)
        self.sunspec_c = QtWidgets.QLineEdit(Dialog)
        self.sunspec_c.setObjectName("sunspec_c")
        self.gridLayout.addWidget(self.sunspec_c, 7, 2, 1, 1)
        self.smdx_list = QtWidgets.QListWidget(Dialog)
        self.smdx_list.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.smdx_list.setObjectName("smdx_list")
        self.gridLayout.addWidget(self.smdx_list, 11, 2, 3, 1)
        self.pick_hierarchy = QtWidgets.QPushButton(Dialog)
        self.pick_hierarchy.setObjectName("pick_hierarchy")
        self.gridLayout.addWidget(self.pick_hierarchy, 1, 3, 1, 1)
        self.label_4 = QtWidgets.QLabel(Dialog)
        self.label_4.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 6, 0, 1, 1)
        self.interface_c_label = QtWidgets.QLabel(Dialog)
        self.interface_c_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.interface_c_label.setObjectName("interface_c_label")
        self.gridLayout.addWidget(self.interface_c_label, 9, 0, 1, 1)
        self.pick_interface_c = QtWidgets.QPushButton(Dialog)
        self.pick_interface_c.setObjectName("pick_interface_c")
        self.gridLayout.addWidget(self.pick_interface_c, 9, 3, 1, 1)
        self.sunspec_c_label = QtWidgets.QLabel(Dialog)
        self.sunspec_c_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sunspec_c_label.setObjectName("sunspec_c_label")
        self.gridLayout.addWidget(self.sunspec_c_label, 7, 0, 1, 1)
        self.interface_c = QtWidgets.QLineEdit(Dialog)
        self.interface_c.setObjectName("interface_c")
        self.gridLayout.addWidget(self.interface_c, 9, 2, 1, 1)
        self.pick_can = QtWidgets.QPushButton(Dialog)
        self.pick_can.setObjectName("pick_can")
        self.gridLayout.addWidget(self.pick_can, 0, 3, 1, 1)
        self.smdx_label = QtWidgets.QLabel(Dialog)
        self.smdx_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.smdx_label.setObjectName("smdx_label")
        self.gridLayout.addWidget(self.smdx_label, 11, 0, 1, 1)
        self.pick_sunspec_c = QtWidgets.QPushButton(Dialog)
        self.pick_sunspec_c.setObjectName("pick_sunspec_c")
        self.gridLayout.addWidget(self.pick_sunspec_c, 7, 3, 1, 1)
        self.sunspec_bitfields_c_label = QtWidgets.QLabel(Dialog)
        self.sunspec_bitfields_c_label.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.sunspec_bitfields_c_label.setObjectName("sunspec_bitfields_c_label")
        self.gridLayout.addWidget(self.sunspec_bitfields_c_label, 4, 0, 1, 1)
        self.sunspec_bitfields_c = QtWidgets.QLineEdit(Dialog)
        self.sunspec_bitfields_c.setObjectName("sunspec_bitfields_c")
        self.gridLayout.addWidget(self.sunspec_bitfields_c, 4, 2, 1, 1)
        self.pick_sunspec_bitfields_c = QtWidgets.QPushButton(Dialog)
        self.pick_sunspec_bitfields_c.setObjectName("pick_sunspec_bitfields_c")
        self.gridLayout.addWidget(self.pick_sunspec_bitfields_c, 4, 3, 1, 1)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Dialog"))
        self.pick_sunspec_tables_c.setText(_translate("Dialog", "..."))
        self.label_2.setText(_translate("Dialog", "Hierarchy:"))
        self.sil_c_label.setText(_translate("Dialog", "SIL C:"))
        self.pick_sil_c.setText(_translate("Dialog", "..."))
        self.label_3.setText(_translate("Dialog", "SunSpec Spreadsheet:"))
        self.tables_c_label.setText(_translate("Dialog", "Tables C:"))
        self.remove_smdx.setText(_translate("Dialog", "Remove"))
        self.pick_spreadsheet.setText(_translate("Dialog", "..."))
        self.pick_smdx.setText(_translate("Dialog", "..."))
        self.pick_spreadsheet_user.setText(_translate("Dialog", "..."))
        self.label.setText(_translate("Dialog", "CAN:"))
        self.sunspec_tables_c_label.setText(_translate("Dialog", "SunSpec Tables C:"))
        self.pick_tables_c.setText(_translate("Dialog", "..."))
        self.from_directory.setText(_translate("Dialog", "From directory..."))
        self.pick_hierarchy.setText(_translate("Dialog", "..."))
        self.label_4.setText(_translate("Dialog", "SunSpec User Spreadsheet:"))
        self.interface_c_label.setText(_translate("Dialog", "Interface C:"))
        self.pick_interface_c.setText(_translate("Dialog", "..."))
        self.sunspec_c_label.setText(_translate("Dialog", "SunSpec C:"))
        self.pick_can.setText(_translate("Dialog", "..."))
        self.smdx_label.setText(_translate("Dialog", "SunSpec SMDX:"))
        self.pick_sunspec_c.setText(_translate("Dialog", "..."))
        self.sunspec_bitfields_c_label.setText(_translate("Dialog", "SunSpec Bitfields C"))
        self.pick_sunspec_bitfields_c.setText(_translate("Dialog", "..."))
